/******************************************************************************
 * @file     main.c
 * @brief
 *           Use Stride mode of PDMA to extract red, green and blue data from
 *           a RGB array.
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 *****************************************************************************/
#include "stdio.h"
#include "NuMicro.h"

/*---------------------------------------------------------------------------*/
/* Definitions                                                               */
/*---------------------------------------------------------------------------*/
#define COLOR_ARRAY_SIZE    4
#define USE_PDMA_CHANNEL    0   // Only channel0-channel5 support stride mode

/*---------------------------------------------------------------------------*/
/* Global variables                                                          */
/*---------------------------------------------------------------------------*/
volatile uint8_t is_pdma_over = 0;

uint8_t rgb[COLOR_ARRAY_SIZE * 3];
uint8_t red[COLOR_ARRAY_SIZE], green[COLOR_ARRAY_SIZE], blue[COLOR_ARRAY_SIZE];
/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/
void SYS_Init(void)
{
    /* Enable HXT clock (external XTAL 12MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Wait for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(FREQ_192MHZ);
    CLK->PCLKDIV = (CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2); // PCLK divider set 2

    /* Enable module clock */
    CLK_EnableModuleClock(UART0_MODULE);
    CLK_EnableModuleClock(PDMA_MODULE);

    /* Set module clock */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HXT, NULL);
}

void UART0_Init()
{
    /* Set PB.12 for UART0_RXD, PB.13 for UART0_TXD */
    SYS->GPB_MFPH &= ~(SYS_GPB_MFPH_PB12MFP_Msk | SYS_GPB_MFPH_PB13MFP_Msk);
    SYS->GPB_MFPH |= (SYS_GPB_MFPH_PB12MFP_UART0_RXD | SYS_GPB_MFPH_PB13MFP_UART0_TXD);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}

int32_t main(void)
{
    uint32_t i, j;

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART0 for printf and test */
    UART0_Init();

    /* Fill up RGB data for test */
    j = 0;
    for(i = 0; i < (COLOR_ARRAY_SIZE * 3); i+=3) {
        rgb[i] = (j + 1);
        j++;
    }

    j = 0;
    for(i = 1; i < (COLOR_ARRAY_SIZE * 3); i+=3) {
        rgb[i] = (j + 0x11);
        j++;
    }

    j = 0;
    for(i = 2; i < (COLOR_ARRAY_SIZE * 3); i+=3) {
        rgb[i] = (j + 0x21);
        j++;
    }

    /* Print RGB pattern */
    printf("RGB Array - \r\n");
    for(i = 0; i < (COLOR_ARRAY_SIZE * 3); i++) {
        printf("0x%02X ", rgb[i]);
    }
    printf("\r\n");

    /* Open PDMA channel and enable interrupt */
    PDMA_Open(PDMA, (1 << USE_PDMA_CHANNEL));
    PDMA_EnableInt(PDMA, USE_PDMA_CHANNEL, PDMA_INT_TRANS_DONE);
    NVIC_EnableIRQ(PDMA_IRQn);

    /* Extract red color from RGB array */
    PDMA_SetTransferCnt(PDMA, USE_PDMA_CHANNEL, PDMA_WIDTH_8, COLOR_ARRAY_SIZE);
    PDMA_SetTransferAddr(PDMA, USE_PDMA_CHANNEL, (uint32_t)rgb, PDMA_SAR_INC, (uint32_t)red, PDMA_DAR_INC);
    PDMA_SetTransferMode(PDMA, USE_PDMA_CHANNEL, PDMA_MEM, FALSE, 0);
    PDMA_SetBurstType(PDMA, USE_PDMA_CHANNEL, PDMA_REQ_BURST, PDMA_BURST_4);
    PDMA_SetStride(PDMA, USE_PDMA_CHANNEL, 1, 3, 1);
    PDMA_Trigger(PDMA, USE_PDMA_CHANNEL);

    /* Wait PDMA transfer done */
    while(!is_pdma_over);
    is_pdma_over = 0;


    /* Extract green color from RGB array */
    PDMA_SetTransferCnt(PDMA, USE_PDMA_CHANNEL, PDMA_WIDTH_8, COLOR_ARRAY_SIZE);
    PDMA_SetTransferAddr(PDMA, USE_PDMA_CHANNEL, (uint32_t)&rgb[1], PDMA_SAR_INC, (uint32_t)green, PDMA_DAR_INC);
    PDMA_SetTransferMode(PDMA, USE_PDMA_CHANNEL, PDMA_MEM, FALSE, 0);
    PDMA_SetBurstType(PDMA, USE_PDMA_CHANNEL, PDMA_REQ_BURST, PDMA_BURST_4);
    PDMA_SetStride(PDMA, USE_PDMA_CHANNEL, 1, 3, 1);
    PDMA_Trigger(PDMA, USE_PDMA_CHANNEL);

    /* Wait PDMA transfer done */
    while(!is_pdma_over);
    is_pdma_over = 0;

    /* Extract blue color from RGB array */
    PDMA_SetTransferCnt(PDMA, USE_PDMA_CHANNEL, PDMA_WIDTH_8, COLOR_ARRAY_SIZE);
    PDMA_SetTransferAddr(PDMA, USE_PDMA_CHANNEL, (uint32_t)&rgb[2], PDMA_SAR_INC, (uint32_t)blue, PDMA_DAR_INC);
    PDMA_SetTransferMode(PDMA, USE_PDMA_CHANNEL, PDMA_MEM, FALSE, 0);
    PDMA_SetBurstType(PDMA, USE_PDMA_CHANNEL, PDMA_REQ_BURST, PDMA_BURST_4);
    PDMA_SetStride(PDMA, USE_PDMA_CHANNEL, 1, 3, 1);
    PDMA_Trigger(PDMA, USE_PDMA_CHANNEL);

    /* Wait PDMA transfer done */
    while(!is_pdma_over);
    is_pdma_over = 0;

    /* Print red, greena and blue data for result */
    printf("Red Data in RGB array - \r\n");
    for(i = 0; i < COLOR_ARRAY_SIZE; i++) {
        printf("0x%02X ", red[i]);
    }
    printf("\r\n");

    printf("Green Data in RGB array - \r\n");
    for(i = 0; i < COLOR_ARRAY_SIZE; i++) {
        printf("0x%02X ", green[i]);
    }
    printf("\r\n");

    printf("Blue Data in RGB array - \r\n");
    for(i = 0; i < COLOR_ARRAY_SIZE; i++) {
        printf("0x%02X ", blue[i]);
    }
    printf("\r\n");

    while(1);
}

void PDMA_IRQHandler(void)
{
    uint32_t status = PDMA_GET_INT_STATUS(PDMA);

    if(status & PDMA_INTSTS_TDIF_Msk) {      /* done */
        if(PDMA_GET_TD_STS(PDMA) & (1 << USE_PDMA_CHANNEL)) {

            is_pdma_over = 1;

            PDMA_CLR_TD_FLAG(PDMA, (1 << USE_PDMA_CHANNEL));
        }
    } else if(status & PDMA_INTSTS_ABTIF_Msk) { /* abort */
        if(PDMA_GET_ABORT_STS(PDMA) & (1 << USE_PDMA_CHANNEL)) {

            printf("PDMA Channel [%d] abort happened\r\n", USE_PDMA_CHANNEL);

            PDMA_CLR_ABORT_FLAG(PDMA, (1 << USE_PDMA_CHANNEL));
        }
    } else
        printf("unknown interrupt !!\n");
}


/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
